@@ -104,6 +104,9 @@ static int cmdhlp(void)
 #endif
 #ifdef HAVE_SDL2
 		printf("     sdl2         : SDL2 video driver\n");
+#endif
+#ifdef HAVE_SDL3
+		printf("     sdl3         : SDL3 video driver\n");
 #endif
 		printf("\nExample : ocp -fl0,r1 -vf2 -spdevpdisk -sr48000 ftstar.xm\n");
 		printf("          (for nice HD rendering of modules)\n");
@@ -753,7 +756,7 @@ static int init_modules(int argc, char *argv[]
 			cfRemoveProfile("filetype 254");
 
 			printf("ocp.ini update (0.2.91) Removed modextensions\n"); /* each plugin registers this */
-			cfRemoveEntry("screen", "modextensions");
+			cfRemoveEntry("fileselector", "modextensions");
 		}
 
 		if (epoch < 20211102)
@@ -1061,9 +1064,27 @@ static int init_modules(int argc, char *argv[]
 			cfSetProfileInt ("fscolors", "QOA", 6, 10);
 		}
 
-		if (epoch < 20260111)
+		if (epoch < 20260223)
 		{
-			cfSetProfileInt("version", "epoch", 20260111, 10);
+			fprintf(stderr, "ocp.ini update (3.2.0) reintroduced [fileselector] modextensions= for manually adding extra file extensions\n");
+			cfSetProfileString ("fileselector", "modextensions", cfGetProfileString("fileselector", "modextensions", ""));
+		}
+
+		if (epoch < 20260227)
+		{
+			const char *temp;
+			fprintf(stderr, "ocp.ini update (3.2.0) libsidplayfp removed emulator=residfpII and added emulator=crSID\n");
+			temp = cfGetProfileString("libsidplayfp", "emulator", "residfp");
+			if (!strcasecmp (temp, "residfpII"))
+			{
+				cfSetProfileString("libsidplayfp", "emulator", "residfp");
+			}
+			cfSetProfileComment ("libsidplayfp", "emulator", "; Possible values are residfp and crSID");
+		}
+
+		if (epoch < 20260227)
+		{
+			cfSetProfileInt("version", "epoch", 20260227, 10);
 			cfStoreConfig();
 #ifdef _WIN32
 			uint16_t *wConfigHomePath = utf8_to_utf16 (ConfigHomePath);
@@ -1073,24 +1094,24 @@ static int init_modules(int argc, char *argv[]
 			if (isatty(2))
 			{
 				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\033[0m\nrm -f ~/.ocp/ocp.ini\033[1m\033[31m or \033[0m\nrm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
+				sleep(5);
 			} else {
 				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\nrm -f ~/.ocp/ocp.ini or rm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
 			}
-			sleep(5);
 #endif
 		}
 	}
-	if (cfGetProfileInt("version", "epoch", 0, 10) != 20260111)
+	if (cfGetProfileInt("version", "epoch", 0, 10) != 20260227)
 	{
 #ifndef _WIN32
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20260111\033[0m\n\n");
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20260227\033[0m\n\n");
 			sleep(5);
 		} else
 #endif
 		{
-			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20260111\n\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20260227\n\n");
 		}
 	}
 
