@@ -519,6 +519,7 @@ class RADPlayer {
 			int8_t          TransposeOctave;
 			int8_t          TransposeNote;
 			uint8_t         LastInstrument;
+			bool            Updated;
 		} Riff, IRiff;
 	};
 
@@ -1611,6 +1612,8 @@ void RADPlayer::ResetFX(CEffects *fx) {
 void RADPlayer::TickRiff(int channum, CChannel::CRiff &riff, bool chan_riff) {
 	uint8_t lineid;
 
+	riff.Updated = true;
+
 	if (riff.SpeedCnt == 0) {
 		ResetFX(&riff.FX);
 		return;
@@ -1632,6 +1635,11 @@ void RADPlayer::TickRiff(int channum, CChannel::CRiff &riff, bool chan_riff) {
 	if (trk && (*trk & 0x7F) == line) {
 		lineid = *trk++;
 
+		// The current riff may be clobbered by recursive riffs.
+		// If this happens, this riff should exit early and not update the
+		// current playing riff pointer (or attempt a line jump).
+		riff.Updated = false;
+
 		if (chan_riff) {
 
 			// Channel riff: play current note
@@ -1654,6 +1662,11 @@ void RADPlayer::TickRiff(int channum, CChannel::CRiff &riff, bool chan_riff) {
 			} while (!last);
 		}
 
+		// Exit if another riff replaced or stopped this riff in the recursive call.
+		if (riff.Updated)
+			return;
+		riff.Updated = true;
+
 		// Last line?
 		if (lineid & 0x80)
 			trk = 0;
