@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'24 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'26 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Glue logic for using libsidplayfp playback engine
  *
@@ -19,14 +19,12 @@
  */
 
 #include "libsidplayfp-api.h"
-#include <iostream>
-#include <fstream>
 #include <vector>
 #include "sidplayfp/sidbuilder.h"
 #include "sidplayfp/SidInfo.h"
 #include "sidplayfp/SidTuneInfo.h"
 #include "builders/residfp-builder/residfp.h"
-#include "builders/resid-builder/resid.h"
+#include "builders/sidlite-builder/sidlite.h"
 
 extern "C"
 {
@@ -47,6 +45,7 @@ extern "C"
 #include <unistd.h>
 #include "types.h"
 #include "boot/psetting.h"
+#include "cpiface/cpiface.h"
 #include "filesel/dirdb.h"
 #include "filesel/filesystem-drive.h"
 #include "filesel/filesystem.h"
@@ -139,12 +138,15 @@ namespace libsidplayfp
 		}
 	}
 
-	ConsolePlayer::ConsolePlayer (const unsigned int rate, const struct configAPI_t *configAPI, const struct dirdbAPI_t *dirdbAPI, struct dmDrive *dmFile) :
+	ConsolePlayer::ConsolePlayer (const unsigned int rate, struct cpifaceSessionAPI_t *cpifaceSession) :
 		m_tune(nullptr),
 		m_state(playerStopped),
 		selected_track(0),
 		sidplayer(*(new libsidplayfp::Player))
 	{
+		const struct configAPI_t *configAPI = cpifaceSession->configAPI;
+		const struct dirdbAPI_t *dirdbAPI = cpifaceSession->dirdb;
+		struct dmDrive *dmFile = cpifaceSession->dmFile;
 		char *endptr;
 
 		m_engCfg = sidplayer.config();
@@ -283,8 +285,10 @@ namespace libsidplayfp
 		m_engCfg.digiBoost = configAPI->GetProfileBool("libsidplayfp", "digiboost", 0, 0);
 		//fprintf (stderr, "digiboost=%d\n", m_engCfg.digiBoost);
 
-		// TODO, add check of this return value
-		createSidEmu (configAPI);
+		if (!createSidEmu (configAPI))
+		{
+			#warning handle this failure
+		}
 
 		const char *kernal_string  = configAPI->GetProfileString("libsidplayfp", "kernal",  "KERNAL.ROM");
 		const char *basic_string   = configAPI->GetProfileString("libsidplayfp", "basic",   "BASIC.ROM");
@@ -304,9 +308,9 @@ namespace libsidplayfp
 		basic_ref   = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, basic_string,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
 		chargen_ref = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, chargen_string, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
 #endif
-		uint8_t *kernalRom  = loadRom ( kernal_ref, 8192, dirdbAPI);
-		uint8_t *basicRom   = loadRom (  basic_ref, 8192, dirdbAPI);
-		uint8_t *chargenRom = loadRom (chargen_ref, 4096, dirdbAPI);
+		uint8_t *kernalRom  = loadRom ( kernal_ref, 8192, cpifaceSession);
+		uint8_t *basicRom   = loadRom (  basic_ref, 8192, cpifaceSession);
+		uint8_t *chargenRom = loadRom (chargen_ref, 4096, cpifaceSession);
 
 		dirdbAPI->Unref ( kernal_ref, dirdb_use_file);
 		dirdbAPI->Unref (  basic_ref, dirdb_use_file);
@@ -327,36 +331,48 @@ namespace libsidplayfp
 		delete &sidplayer;
 	}
 
-	uint8_t* ConsolePlayer::loadRom(uint32_t dirdb_ref, const int size, const struct dirdbAPI_t *dirdbAPI)
+	uint8_t* ConsolePlayer::loadRom(uint32_t dirdb_ref, const int size, struct cpifaceSessionAPI_t *cpifaceSession)
 	{
 		char *romPath = 0;
 #ifdef _WIN32
-		dirdbAPI->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_DRIVE | DIRDB_FULLNAME_BACKSLASH);
+		cpifaceSession->dirdb->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_DRIVE | DIRDB_FULLNAME_BACKSLASH);
 #else
-		dirdbAPI->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_NODRIVE);
+		cpifaceSession->dirdb->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_NODRIVE);
 #endif
-		std::ifstream is(romPath, std::ios::binary);
-
-		if (is.is_open())
+		FILE *f;
+#ifdef _WIN32
+		uint16_t *name = cpifaceSession->utf8_to_utf16_LFN (romPath, 0);
+		if (!name)
 		{
-			try
-			{
-				uint8_t *buffer = new uint8_t[size];
-
-				is.read((char*)buffer, size);
-				if (!is.fail())
-				{
-					is.close();
-					return buffer;
-				}
-				delete [] buffer;
-			}
-			catch (std::bad_alloc const &ba) {}
+			cpifaceSession->cpiDebug (cpifaceSession, "ConsolePlayer::loadRom: utf8_to_utf16_LFN(\"%s\") failed\n", romPath);
+			free (romPath);
+			return nullptr;
+		}
+		f = _wfopen ((wchar_t *)name, L"rb");
+		free (name);
+#else
+		f = fopen (romPath, "rb");
+#endif
+		if (!f)
+		{
+			cpifaceSession->cpiDebug (cpifaceSession, "ConsolePlayer::loadRom: fopen(\"%s\") failed\n", romPath);
+			free (romPath);
+			return nullptr;
 		}
 
-		free (romPath);
+		uint8_t *buffer = new uint8_t[size];
+		if (fread (buffer, 1, size, f) != size)
+		{
+			fclose (f);
+			delete [] buffer;
+			cpifaceSession->cpiDebug (cpifaceSession, "ConsolePlayer::loadRom: fread (\"%s\", %d) failed\n", romPath, (int)size);
+			free (romPath);
+			return nullptr;
+		}
 
-		return nullptr;
+		fclose (f);
+		free (romPath);
+		return buffer;
 	}
 
 	void ConsolePlayer::clearSidEmu (void)
@@ -377,48 +393,27 @@ namespace libsidplayfp
 		// Remove old driver and emulation - should be no-op
 		clearSidEmu ();
 
-		int use_residfp = !strcmp(configAPI->GetProfileString("libsidplayfp", "emulator", "residfp"), "residfp");
+		int use_residfp = !strcmp(configAPI->GetProfileString("libsidplayfp", "emulator", "residfp"), "residfp"); // true for residfp, false for crSID
 
 		//fprintf (stderr, "use_residfp=%d\n", use_residfp);
 
 		try
 		{
-			if (use_residfp)
+			if (!use_residfp)
 			{
+				m_residfp = 2;
+				SIDLiteBuilder *rs = new SIDLiteBuilder ( "crSID" );
+				m_engCfg.sidEmulation = rs;
+				/* TODO: no parameters available in crSID (sidlite) */
+			} else {
+				m_residfp = 1;
 				ReSIDfpBuilder *rs = new ReSIDfpBuilder( "ReSIDFP" );
-
 				m_engCfg.sidEmulation = rs;
-				if (!rs->getStatus())
-				{
-					goto createSidEmu_error;
-				}
-
-				rs->create ((sidplayer.info()).maxsids());
-				if (!rs->getStatus())
-				{
-					goto createSidEmu_error;
-				}
-
 				rs->filter6581Curve(m_filter.filterCurve6581);
 				rs->filter6581Range(m_filter.filterRange6581);
 				rs->filter8580Curve(m_filter.filterCurve8580);
 				rs->combinedWaveformsStrength(m_filter.combinedWaveforms);
-			} else {
-				ReSIDBuilder *rs = new ReSIDBuilder( "ReSID" );
 
-				m_engCfg.sidEmulation = rs;
-				if (!rs->getStatus())
-				{
-					goto createSidEmu_error;
-				}
-
-				rs->create ((sidplayer.info ()).maxsids());
-				if (!rs->getStatus())
-				{
-					goto createSidEmu_error;
-				}
-
-				rs->bias(m_filter.bias);
 			}
 		}
 		catch (std::bad_alloc const &ba) {}
@@ -459,7 +454,9 @@ namespace libsidplayfp
 			return false;
 		}
 
-		return open();
+		bool retval = open();
+
+		return retval;
 	}
 
 
@@ -479,6 +476,9 @@ namespace libsidplayfp
 			return false;
 		}
 
+		// First now we know number of SID chips. Since sidplayer.load() resets the buffers, so need to perform initMixer here, and not in load()
+		sidplayer.initMixer(true); // false = mono, true = stereo
+
 		m_state = playerRunning;
 
 		return true;
@@ -494,34 +494,33 @@ namespace libsidplayfp
 			{
 				selected_track = 1;
 			}
-			sidplayer.stop ();
+//			sidplayer.stop ();
 			return open ();
 		}
 		return false;
 	}
 
-	bool ConsolePlayer::iterateaudio (int16_t *targetBuffer, uint_least32_t count, std::vector<int16_t *> *rawSamples)
+
+	uint_least32_t ConsolePlayer::iterateaudio (int16_t *targetBuffer, uint_least32_t count, uint_least32_t cycles, std::vector<int16_t *> *rawSamples)
 	{
 		if (m_state == playerRunning)
 		{
-			if (sidplayer.play (targetBuffer, count*2, rawSamples) < 0)
+			uint_least32_t samples_available = sidplayer.play (cycles);
+			if (samples_available > count)
 			{
-				if (sidplayer.isPlaying ())
-				{
-					m_state = playerError;
-				}
-				return false;
+				fprintf (stderr, "[sidplayer] Warning: sample count %u > samples available %u\n", (unsigned int)samples_available, (unsigned int)count);
+				samples_available = count;
 			}
-			return true;
+			return sidplayer.mix(targetBuffer, samples_available, rawSamples);
 		}
-		return false;
+		return 0;
 	}
 
 	void ConsolePlayer::close (void)
 	{
 		if (m_state != playerStopped)
 		{
-			sidplayer.stop ();
+			//sidplayer.stop ();
 		}
 		m_state = playerStopped;
 		clearSidEmu ();
@@ -548,27 +547,22 @@ namespace libsidplayfp
 		sidplayer.filter(2, enable);
 	}
 
-	void ConsolePlayer::SetBias(double bias)
-	{
-		if (bias >  500.0) bias =  500.0;
-		if (bias < -500.0) bias = -500.0;
-
-		ReSIDBuilder *rs = dynamic_cast<ReSIDBuilder *>(m_engCfg.sidEmulation);
-		if (rs)
-		{
-			rs->bias (bias);
-		}
-	}
-
 	void ConsolePlayer::SetFilterCurve6581 (double v)
 	{
 		if (v > 1.0) v = 1.0;
 		if (v < 0.0) v = 0.0;
 
-		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
-		if (rs)
+		if (m_residfp == 1)
 		{
-			rs->filter6581Curve (v);
+			ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+			if (rs)
+			{
+				rs->filter6581Curve (v);
+			}
+		} else if (m_residfp == 2)
+		{
+			SIDLiteBuilder *rs = dynamic_cast<SIDLiteBuilder *>(m_engCfg.sidEmulation);
+			/* TODO: no parameters available in crSID (sidlite) */
 		}
 	}
 
@@ -577,10 +571,17 @@ namespace libsidplayfp
 		if (v > 1.0) v = 1.0;
 		if (v < 0.0) v = 0.0;
 
-		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
-		if (rs)
+		if (m_residfp == 1)
 		{
-			rs->filter6581Range (v);
+			ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+			if (rs)
+			{
+				rs->filter6581Range (v);
+			}
+		} else if (m_residfp == 2)
+		{
+			SIDLiteBuilder *rs = dynamic_cast<SIDLiteBuilder *>(m_engCfg.sidEmulation);
+			/* TODO: no parameters available in crSID (sidlite) */
 		}
 	}
 
@@ -589,10 +590,17 @@ namespace libsidplayfp
 		if (v > 1.0) v = 1.0;
 		if (v < 0.0) v = 0.0;
 
-		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
-		if (rs)
+		if (m_residfp == 1)
 		{
-			rs->filter8580Curve (v);
+			ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+			if (rs)
+			{
+				rs->filter8580Curve (v);
+			}
+		} else if (m_residfp == 2)
+		{
+			SIDLiteBuilder *rs = dynamic_cast<SIDLiteBuilder *>(m_engCfg.sidEmulation);
+			/* TODO: no parameters available in crSID (sidlite) */
 		}
 	}
 
@@ -600,12 +608,20 @@ namespace libsidplayfp
 	{
 		if (CWS < 0) CWS = 0;
 		if (CWS > 2) CWS = 2;
-		ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
-		if (rs)
+
+		if (m_residfp == 1)
 		{
-			if (CWS == 0) rs->combinedWaveformsStrength (SidConfig::AVERAGE);
-			if (CWS == 1) rs->combinedWaveformsStrength (SidConfig::WEAK);
-			if (CWS == 2) rs->combinedWaveformsStrength (SidConfig::STRONG);
+			ReSIDfpBuilder *rs = dynamic_cast<ReSIDfpBuilder *>(m_engCfg.sidEmulation);
+			if (rs)
+			{
+				if (CWS == 0) rs->combinedWaveformsStrength (SidConfig::AVERAGE);
+				if (CWS == 1) rs->combinedWaveformsStrength (SidConfig::WEAK);
+				if (CWS == 2) rs->combinedWaveformsStrength (SidConfig::STRONG);
+			}
+		} else if (m_residfp == 2)
+		{
+			SIDLiteBuilder *rs = dynamic_cast<SIDLiteBuilder *>(m_engCfg.sidEmulation);
+			/* TODO: no parameters available in crSID (sidlite) */
 		}
 	}
 
@@ -638,6 +654,13 @@ namespace libsidplayfp
 		return modelData[sidplayer.getModel()].vicModel;
 	}
 
+	const uint32_t ConsolePlayer::GetVICIICyclesPerFrame(void)
+	{
+		c64vic vic = sidplayer.getVIC();
+		return vic.maxRasters *
+		       vic.cyclesPerLine;
+	}
+
 	const char *ConsolePlayer::getCIAmodel(void)
 	{
 		const SidConfig &config = sidplayer.config ();
