@@ -23,10 +23,7 @@
 #ifndef SIDPLAYFP_H
 #define SIDPLAYFP_H
 
-#include <stdint.h>
-#include <stdio.h>
 #include <vector>
-
 #include "sidplayfp/siddefs.h"
 #include "sidplayfp/sidversion.h"
 
@@ -131,7 +128,7 @@ public:
     unsigned int installedSIDs() const;
 
     /**
-     * Init mixer.
+     * Initialize the mixer.
      *
      * @param stereo whether to mix in stereo or mono
      * @since 2.15
@@ -149,9 +146,8 @@ public:
     unsigned int mix(short *buffer, unsigned int samples);
 
     /**
-     * Control debugging.
-     * Only has effect if library have been compiled
-     * with the --enable-debug option.
+     * Control CPU tracing.
+     * @note: Must be called before #reset
      *
      * @param enable enable/disable debugging.
      * @param out the file where to redirect the debug info.
@@ -169,7 +165,7 @@ public:
 
     /**
      * Enable/disable SID filter.
-     * Must be called after #config or it has no effect.
+     * @note: Must be called after #config or it has no effect.
      *
      * @param sidNum the SID chip, 0 for the first one, 1 for the second or 2 for the third.
      * @param enable true enable the filter, false disable it.
