@@ -315,7 +315,7 @@ static void sidConfigDraw (int EditPos, const struct DevInterfaceAPI_t *API)
 	      int mlHeight = contentheight + LINES_NOT_AVAILABLE;
 	      int mlTop, mlLeft, mlWidth;
 	const char *offon[] = {"off", "on"};
-	const char *emulators[] = {"residfp", "residfpII"};
+	const char *emulators[] = {"residfp", "crSID"};
 	const char *C64models[] = {"PAL", "NTSC", "OLD-NTSC", "DREAN", "PAL-M"};
 	const char *SIDmodels[] = {"MOS6581", "MOS8580"};
 	const char *CIAmodels[] = {"MOS6526", "MOS6526W4485", "MOS8521"};
@@ -398,8 +398,8 @@ if (skip)                   \
 	switch (EditPos)
 	{
 		case 0:
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfp   = floating point emulator.");
-			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfpII = floating point emulator, mark II.");
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " residfp = floating point emulator.");
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " crSID   = fixed point emulator (sidlite)");
 			break;
 		case 1:
 			API->console->DisplayPrintf (mlTop++, mlLeft, 0x07, mlWidth, " For SID files that does not specify C64 model.");
@@ -467,7 +467,8 @@ if (skip)                   \
 static int emulator_to_int (const char *src)
 {
 	if (!strcasecmp (src, "residfp")) return 0;
-	if (!strcasecmp (src, "residfpII")) return 1;
+	if (!strcasecmp (src, "crSID")) return 1;
+
 	return 1;
 }
 
@@ -475,9 +476,9 @@ static const char *emulator_from_int (const int src)
 {
 	switch (src)
 	{
-		case 0: return "residfp";
 		default:
-		case 1: return "residfpII";
+		case 0: return "residfp";
+		case 1: return "crSID";
 	}
 }
 
@@ -1057,9 +1058,15 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 	if (config_filtercurve8580 < 0) config_filtercurve8580 = 0;
 	if (config_filtercurve8580 > 100) config_filtercurve8580 = 100;
 
-	entry_kernal.dirdb_ref  = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_kernal,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
-	entry_basic.dirdb_ref   = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_basic,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
-	entry_chargen.dirdb_ref = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_chargen, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+#ifdef _WIN32
+	entry_kernal.dirdb_ref  = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_kernal,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH, dirdb_use_file);
+	entry_basic.dirdb_ref   = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_basic,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH, dirdb_use_file);
+	entry_chargen.dirdb_ref = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_chargen, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH, dirdb_use_file);
+#else
+	entry_kernal.dirdb_ref  = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_kernal,  DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+	entry_basic.dirdb_ref   = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_basic,   DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+	entry_chargen.dirdb_ref = API->dirdb->ResolvePathWithBaseAndRef (dirdb_base, config_chargen, DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+#endif
 
 	rom_md5s (entry_kernal .hash_4096, entry_kernal .hash_8192, entry_kernal .dirdb_ref, API);
 	rom_md5s (entry_basic  .hash_4096, entry_basic  .hash_8192, entry_basic  .dirdb_ref, API);
@@ -1197,8 +1204,13 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 											char *newpath = 0;
 											char *config = 0;
 											char *path = 0;
+#ifdef _WIN32
+											API->dirdb->GetFullname_malloc (entries_data[dsel].dirdb_ref, &path, DIRDB_FULLNAME_DRIVE | DIRDB_FULLNAME_BACKSLASH);
+											API->dirdb->GetFullname_malloc (dirdb_base, &config, DIRDB_FULLNAME_DRIVE | DIRDB_FULLNAME_ENDSLASH | DIRDB_FULLNAME_BACKSLASH);
+#else
 											API->dirdb->GetFullname_malloc (entries_data[dsel].dirdb_ref, &path, DIRDB_FULLNAME_NODRIVE);
 											API->dirdb->GetFullname_malloc (dirdb_base, &config, DIRDB_FULLNAME_NODRIVE | DIRDB_FULLNAME_ENDSLASH);
+#endif
 											if (!strncmp (path, config, strlen (config)))
 											{
 												newpath = malloc (1 + strlen (path) - strlen (config));
@@ -1215,7 +1227,11 @@ static void sidConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 												newpath = malloc (3 + strlen (path) - strlen (API->configAPI->HomePath));
 												if (newpath)
 												{
+#ifdef _WIN32
+													sprintf (newpath, "~\%s", path + strlen (API->configAPI->HomePath));
+#else
 													sprintf (newpath, "~/%s", path + strlen (API->configAPI->HomePath));
+#endif
 												}
 											} else {
 												newpath = path;
