@@ -531,12 +531,11 @@ static void sidSetPitch (uint32_t sp)
 
 static void sidSetVolume (void)
 {
-	voll=vol*4;
-	volr=vol*4;
-	if (bal<0)
-		volr=(volr*(64+bal))>>6;
+	volr = voll = vol * 4;
+	if (bal < 0)
+		voll = (voll * (64 + bal)) >> 6;
 	else
-		voll=(voll*(64-bal))>>6;
+		volr = (volr * (64 - bal)) >> 6;
 }
 
 static void sidSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
@@ -941,7 +940,7 @@ OCP_INTERNAL int sidOpenPlayer (struct ocpfilehandle_t *file, struct cpifaceSess
 		goto error_out_buf;
 	}
 
-	mySidPlayer = new libsidplayfp::ConsolePlayer(sidRate, cpifaceSession->configAPI, cpifaceSession->dirdb, cpifaceSession->dmFile);
+	mySidPlayer = new libsidplayfp::ConsolePlayer(sidRate, cpifaceSession);
 	if (!mySidPlayer->load (buf, length))
 	{
 		cpifaceSession->cpiDebug (cpifaceSession, "[SID] loading file failed\n");
