@@ -148,11 +148,11 @@ static void ymSetSpeed(uint16_t sp)
 
 static void ymSetVolume(void)
 {
-	volr=voll=vol*4;
-	if (bal<0)
-		volr=(volr*(64+bal))>>6;
+	volr = voll = vol * 4;
+	if (bal < 0)
+		voll = (voll * (64 + bal)) >> 6;
 	else
-		voll=(voll*(64-bal))>>6;
+		volr = (volr * (64 - bal)) >> 6;
 }
 
 static void ymSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
