@@ -425,6 +425,13 @@ static void WindowResized_Graphmode(unsigned int width, unsigned int height)
 
 static void WindowResized_Textmode(unsigned int width, unsigned int height)
 {
+	const int minwidth  = CONSOLE_MIN_X * 8;
+	const int minheight = CONSOLE_MIN_Y * 8;
+
+	/* Fake a bigger console, if XSetWMNormalHints does not take effect (Hyprland) */
+	if (width  < minwidth)  width  = minwidth;
+	if (height < minheight) height = minheight;
+
 	Console.GraphBytesPerLine = width;
 	Console.GraphLines = height;
 
@@ -432,12 +439,12 @@ static void WindowResized_Textmode(unsigned int width, unsigned int height)
 
 	if (Console.CurrentFont >= _16x32)
 	{
-		if ( ( Console.GraphBytesPerLine < ( 16 * 80 ) ) || ( Console.GraphLines < ( 32 * 25 ) ) )
+		if ( ( Console.GraphBytesPerLine < ( 16 * CONSOLE_MIN_X ) ) || ( Console.GraphLines < ( 32 * CONSOLE_MIN_Y ) ) )
 			Console.CurrentFont = _8x16;
 	}
 	if (Console.CurrentFont >= _8x16)
 	{
-		if ( ( Console.GraphBytesPerLine < ( 8 * 80 ) ) || ( Console.GraphLines < ( 16 * 25 ) ) )
+		if ( ( Console.GraphBytesPerLine < ( 8 * CONSOLE_MIN_X ) ) || ( Console.GraphLines < ( 16 * CONSOLE_MIN_Y ) ) )
 			Console.CurrentFont = _8x8;
 	}
 
@@ -626,20 +633,20 @@ static void TextModeSetState(FontSizeEnum FontSize, int FullScreen)
 	{
 		/* find the min value, if screen is too small, force smaller font */
 		case _16x32:
-			Textmode_SizeHints.min_width = 16 * 80;
-			Textmode_SizeHints.min_height = 32 * 25;
+			Textmode_SizeHints.min_width = 16 * CONSOLE_MIN_X;
+			Textmode_SizeHints.min_height = 32 * CONSOLE_MIN_Y;
 			if ( (Textmode_SizeHints.min_width <= Textmode_SizeHints.max_width) && (Textmode_SizeHints.min_height <= Textmode_SizeHints.max_height) )
 				break;
 			FontSize = _8x16; /* drop through */
 		case _8x16:
-			Textmode_SizeHints.min_width = 8 * 80;
-			Textmode_SizeHints.min_height = 16 * 25;
+			Textmode_SizeHints.min_width = 8 * CONSOLE_MIN_X;
+			Textmode_SizeHints.min_height = 16 * CONSOLE_MIN_Y;
 			if ( (Textmode_SizeHints.min_width <= Textmode_SizeHints.max_width) && (Textmode_SizeHints.min_height <= Textmode_SizeHints.max_height) )
 				break;
 			FontSize = _8x8; /* drop through */
 		case _8x8:
-			Textmode_SizeHints.min_width = 8 * 80;
-			Textmode_SizeHints.min_height = 8 * 25;
+			Textmode_SizeHints.min_width = 8 * CONSOLE_MIN_X;
+			Textmode_SizeHints.min_height = 8 * CONSOLE_MIN_Y;
 			if ( (Textmode_SizeHints.min_width <= Textmode_SizeHints.max_width) && (Textmode_SizeHints.min_height <= Textmode_SizeHints.max_height) )
 				break;
 			/* unless font has became this small, force bigger window */
@@ -1503,12 +1510,14 @@ static void x11_SetTextMode (unsigned char x)
 
 #else
 	/* if invalid mode, set it to zero */
-	if (x>=8)
+
+	Console.CurrentFont = x11_CurrentFontWanted;
+	if (x>7)
 	{
 		x=8;
 		Console.TextHeight = Textmode_Window_Height / (x11_CurrentFontWanted == _16x32 ) ? 32 : (x11_CurrentFontWanted == _8x16) ? 16 : 8;
 		Console.TextWidth = Textmode_Window_Width / (x11_CurrentFontWanted == _16x32 ? 16 : 8);
-;
+
 		Console.GraphBytesPerLine = Textmode_Window_Width;
 		Console.GraphLines = Textmode_Window_Height;
 	} else {
@@ -1518,7 +1527,7 @@ static void x11_SetTextMode (unsigned char x)
 		Console.GraphBytesPerLine = modes[x].windowx;
 		Console.GraphLines = modes[x].windowy;
 
-		x11_CurrentFontWanted = modes[x].bigfont ? _8x16 : _8x8;
+		Console.CurrentFont = modes[x].bigfont ? _8x16 : _8x8;
 		Textmode_Window_Width = Console.TextWidth * 8;
 		Textmode_Window_Height = Console.TextHeight * (modes[x].bigfont ? 16 : 8);
 	}
@@ -1530,7 +1539,7 @@ static void x11_SetTextMode (unsigned char x)
 		create_window();
 	}
 
-	TextModeSetState(x11_CurrentFontWanted, do_fullscreen);
+	TextModeSetState(Console.CurrentFont, do_fullscreen);
 
 	___push_key(VIRT_KEY_RESIZE);
 
@@ -2219,19 +2228,16 @@ static void x11_DisplaySetupTextMode (void)
 	while (1)
 	{
 		uint16_t c;
-
 		memset (Console.VidMem, 0, Console.GraphBytesPerLine * Console.GraphLines);
-
 		make_title("x11-driver setup", 0);
 		swtext_displaystr_cp437 (1, 0, 0x07, "1:  font-size:", 14);
-		swtext_displaystr_cp437 (1, 15, Console.CurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
-		swtext_displaystr_cp437 (1, 19, Console.CurrentFont == _8x16 ? 0x0f : 0x07, "8x16", 4);
-		swtext_displaystr_cp437 (1, 24, Console.CurrentFont == _16x32 ? 0x0f : 0x07, "16x32", 5);
+		swtext_displaystr_cp437 (1, 15, Console.CurrentFont == _8x8   ? 0x0f : x11_CurrentFontWanted == _8x8   ? 0x02 : 0x07, "8x8", 3);
+		swtext_displaystr_cp437 (1, 19, Console.CurrentFont == _8x16  ? 0x0f : x11_CurrentFontWanted == _8x16  ? 0x02 : 0x07, "8x16", 4);
+		swtext_displaystr_cp437 (1, 24, Console.CurrentFont == _16x32 ? 0x0f : x11_CurrentFontWanted == _16x32 ? 0x02 : 0x07, "16x32", 5);
 /*
 		swtext_displaystr_cp437 (2, 0, 0x07, "2:  fullscreen: ", 16);
 		swtext_displaystr_cp437 (3, 0, 0x07, "3:  resolution in fullscreen:", 29);
 */
-
 		swtext_displaystr_cp437 (Console.TextHeight - 1, 0, 0x17, "  press the number of the item you wish to change and ESC when done", Console.TextWidth);
 
 		while (! ekbhit())
@@ -2247,7 +2253,6 @@ static void x11_DisplaySetupTextMode (void)
 				/* we can assume that we are in text-mode if we are here */
 				x11_CurrentFontWanted = (x11_CurrentFontWanted == _8x8) ? _8x16 : (x11_CurrentFontWanted == _8x16) ? _16x32 : _8x8;
 				TextModeSetState (x11_CurrentFontWanted, do_fullscreen);
-				x11_CurrentFontWanted = Console.CurrentFont;
 				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
 				break;
 			case KEY_EXIT:
